package de.ullisroboterseite.ursai2applauncher;

import java.util.*;

import javax.naming.spi.DirStateFactory.Result;

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.annotations.androidmanifest.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;
import com.google.appinventor.components.runtime.util.*;

import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;

import android.content.*;

import android.app.Activity;

import android.util.Log;
import java.util.Set;

// Der Übersichtlichkeit halber
// Klasse zum Starten von App
class AppLauncherUtil {
    static final String LOG_TAG = UrsAI2AppLauncher.LOG_TAG;
    final Activity activity;
    final PackageManager pm;
    final int requestCode;

    private class BuildResult {
        int result;
        Intent intent;

        BuildResult(Intent i, int r) {
            intent = i;
            result = r;
        }
    }

    AppLauncherUtil(Activity activity, int requestCode) {
        this.activity = activity;
        pm = activity.getPackageManager();
        this.requestCode = requestCode;
    }

    // #region Build Intent =========================================
    BuildResult buildIntent(String PackageName) {
        Intent mIntent = pm.getLaunchIntentForPackage(PackageName);
        if (mIntent == null) {
            return new BuildResult(null, 1);
        }
        return new BuildResult(mIntent, 0);
    }

    BuildResult buildIntent(String PackageName, String Key, String Value) {
        Intent mIntent = pm.getLaunchIntentForPackage(PackageName);
        if (mIntent == null) {
            return new BuildResult(null, 1);
        }

        if (!Key.isEmpty() && !Value.isEmpty())
            mIntent.putExtra(Key, Value);

        return new BuildResult(mIntent, 0);
    }

    BuildResult buildIntent(String PackageName, YailList Pairs) {
        Intent mIntent = pm.getLaunchIntentForPackage(PackageName);
        if (mIntent == null) {
            return new BuildResult(null, 1);
        }

        for (Object pair : Pairs.toArray()) {
            boolean isYailList = pair instanceof YailList;
            boolean isPair = isYailList ? ((YailList) pair).size() == 2 : false;
            if (!isYailList || !isPair) {
                return new BuildResult(null, 2);
            }
        }

        for (Object extra : Pairs.toArray()) {
            YailList castExtra = (YailList) extra;
            String key = castExtra.getString(0);
            String value = castExtra.getString(1);
            if (key.length() != 0 && value.length() != 0) {
                mIntent.putExtra(key, value);
            }
        }
        return new BuildResult(mIntent, 0);
    }

    // #endregion

    // #region Start Activity =======================================
    int startActivity(BuildResult br, boolean asSubTask) {
        if (br.result != 0)
            return br.result;

        if (!asSubTask) {
            br.intent.setAction(Intent.ACTION_MAIN);
            // Flags und Kategorien löschen
            // Bei den Standardwerten werden keine Werte von der aufgerufenen App zurück
            // geliefert (Android Oreo 8.1.0)
            br.intent.setFlags(0);
            Set<String> cats = br.intent.getCategories();
            if (cats != null)
                for (String temp : cats) {
                    br.intent.removeCategory(temp);
                }
        }

        try {
            activity.startActivityForResult(br.intent, requestCode);
            return 0;
        } catch (ActivityNotFoundException err) {
            return 1;
        }
    }

    public int launchApp(String PackageName, boolean asSubTask) {
        return startActivity(buildIntent(PackageName), asSubTask);
    }

    public int launchAppExtra(String PackageName, String Key, String Value, boolean asSubTask) {
        return startActivity(buildIntent(PackageName, Key, Value), asSubTask);
    }

    public int launchAppExtraList(String PackageName, YailList Pairs, boolean asSubTask) {
        return startActivity(buildIntent(PackageName, Pairs), asSubTask);
    }

    public int launchAppFromIntent(Intent launchIntent) {
        try {
            activity.startActivity(launchIntent);
            return 0;
        } catch (ActivityNotFoundException err) {
            return 1;
        }
    }

    public int launchAppFromIntentForResult(Intent launchIntent) {
        try {
            activity.startActivityForResult(launchIntent,  requestCode);
            return 0;
        } catch (ActivityNotFoundException err) {
            return 1;
        }
    }
    // #endregion
}